#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/__init__.py
# VERSION:     0.1.7
# CREATED:     2024-03-21 13:31
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION: https://www.w3docs.com/snippets/python/what-is-init-py-for.html
#
# HISTORY:
# *************************************************************
"""Module indicating that `~~/tests` directory is present in is a Python package."""

### Standard packages ###
from json import loads
from multiprocessing import Process
from urllib.parse import parse_qs
from time import sleep
from typing import Generator

### Third-party packages ###
from fastapi import FastAPI
from fastapi.requests import Request
from fastapi.responses import JSONResponse, PlainTextResponse, ORJSONResponse
from pytest import fixture
from uvicorn import run

RUNS: int = 1_000
TEST_HOST: str = "localhost"
TEST_PORT: int = 6969
TEST_ENDPOINT: str = f"http://{ TEST_HOST }:{ TEST_PORT }"


def run_test_server() -> None:
  app: FastAPI = FastAPI()

  @app.get("/plaintext", response_class=PlainTextResponse)
  async def plaintext_endpoint() -> str:
    return "OK"

  @app.get("/json", response_class=JSONResponse)
  async def json_endpoint(request: Request) -> dict:
    response = {
      "detail": "OK",
      "float": 1.234,
      "integer": 200,
      "null": None,
    }
    if len(request.query_params) > 0:
      response["query"] = parse_qs(str(request.query_params))
    return response

  @app.get("/orjson", response_class=ORJSONResponse)
  async def orjson_endpoint(request: Request) -> dict:
    response = {
      "detail": "OK",
      "float": 1.234,
      "integer": 200,
      "null": None,
    }
    if len(request.query_params) > 0:
      response["query"] = parse_qs(str(request.query_params))
    return response

  @app.post("/create", response_class=JSONResponse, status_code=201)
  async def create_post(request: Request) -> dict:
    return {"created": loads(await request.json())}

  @app.put("/update", response_class=JSONResponse)
  async def update_post(request: Request) -> dict:
    return {"updated": loads(await request.json())}

  @app.delete("/delete", response_class=PlainTextResponse, status_code=204)
  async def delete_post() -> None: ...

  run(app, host="localhost", port=6969)


@fixture(scope="session", autouse=True)
def setup_teardown_api_server() -> Generator:
  """
  Sets up a FastAPI server with a generic get-endpoint

  ---
  :returns: Generator
  """
  process: Process = Process(daemon=True, target=run_test_server)
  process.start()
  sleep(0.5)  # setup-delay
  yield
  process.terminate()


__all__ = ("RUNS", "TEST_ENDPOINT", "setup_teardown_api_server")



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/conftest.py
# VERSION:     0.1.7
# CREATED:     2024-03-21 16:42
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import List

### Third-party packages ###
from pytest import Config, Item, Parser, mark


def pytest_addoption(parser: Parser) -> None:
  """Add option to Pytest CLI parser

  ---
  :param Parser parser: pytest Parser instance passed when running via pytest cli
  """
  parser.addoption(
    "--bench-against-aiohttp",
    action="store_true",
    default=False,
    help="Include aiohttp when running benchmarks",
  )
  parser.addoption(
    "--bench-against-httpx",
    action="store_true",
    default=False,
    help="Include httpx when running benchmarks",
  )
  parser.addoption(
    "--bench-against-requests",
    action="store_true",
    default=False,
    help="Include requests when running benchmarks",
  )


def pytest_collection_modifyitems(config: Config, items: List[Item]) -> None:
  for item in items:
    if "aiohttp" in item.keywords and not config.getoption("--bench-against-aiohttp"):
      item.add_marker(
        mark.skip(
          reason="Test excluded without flag; Add --bench-against-aiohttp flag to run test."
        )
      )
    elif "httpx" in item.keywords and not config.getoption("--bench-against-httpx"):
      item.add_marker(
        mark.skip(reason="Test excluded without flag; Add --bench-against-httpx flag to run test.")
      )
    elif "requests" in item.keywords and not config.getoption("--bench-against-requests"):
      item.add_marker(
        mark.skip(
          reason="Test excluded without flag; Add --bench-against-requests flag to run test."
        )
      )


__all__ = ("pytest_addoption", "pytest_collection_modifyitems")



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/alcedo.pyi
# VERSION:     0.1.7
# CREATED:     2024-03-21 00:20
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import Optional, Union

class Client:
  def __init__(
    self, headers: Optional[dict] = None, **kwargs: Union[bool, bytes, float, int, str]
  ) -> None: ...
  def delete(
    self, url: str, headers: Optional[dict] = None, payload: dict = {}, query: Optional[dict] = None
  ) -> Response: ...
  def get(
    self, url: str, headers: Optional[dict] = None, query: Optional[dict] = None
  ) -> Response: ...
  def post(
    self, url: str, headers: Optional[dict] = None, payload: dict = {}, query: Optional[dict] = None
  ) -> Response: ...
  def put(
    self, url: str, headers: Optional[dict] = None, payload: dict = {}, query: Optional[dict] = None
  ) -> Response: ...
  def request(
    self, method: str, url: str, headers: Optional[dict] = None, query: Optional[dict] = None
  ) -> Response: ...

class Response:
  def __init__(self, **kwargs: Union[bool, bytes, float, int, str]) -> None: ...
  def json(self) -> dict: ...
  def text(self) -> str: ...
  def raise_for_status(self) -> None: ...
  def status(self) -> int: ...
  def headers(self) -> dict: ...

def get(url: str, **kwargs: Any) -> Response: ...



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/session/queried_get/bench_httpx.py
# VERSION:     0.1.7
# CREATED:     2024-04-01 08:07
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Third-party packages ###
from httpx import Client, Response
from pytest import mark

### Local modules ###
from tests import *


@mark.httpx
def test_httpx_client_get_json_endpoint() -> None:
  client: Client = Client()
  for i in range(RUNS):
    response: Response = client.get(f"{ TEST_ENDPOINT }/json", params={"foo": "bar", "index": i})
    assert response.json() == {
      "detail": "OK",
      "float": 1.234,
      "integer": 200,
      "null": None,
      "query": {"foo": ["bar"], "index": [f"{ i }"]},
    }


@mark.httpx
def test_httpx_client_get_orjson_endpoint() -> None:
  client: Client = Client()
  for i in range(RUNS):
    response: Response = client.get(f"{ TEST_ENDPOINT }/orjson", params={"foo": "bar", "index": i})
    assert response.json() == {
      "detail": "OK",
      "float": 1.234,
      "integer": 200,
      "null": None,
      "query": {"foo": ["bar"], "index": [f"{ i }"]},
    }


@mark.httpx
def test_httpx_client_get_plaintext_endpoint() -> None:
  client: Client = Client()
  for _ in range(RUNS):
    response: Response = client.get(f"{ TEST_ENDPOINT }/plaintext")
    assert response.text == "OK"



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/session/delete/__init__.py
# VERSION:     0.1.7
# CREATED:     2024-03-28 14:24
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION: https://www.w3docs.com/snippets/python/what-is-init-py-for.html
#
# HISTORY:
# *************************************************************
"""Module indicating that `~~/tests/session/delete` directory is present in is a Python package."""



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/get/bench.py
# VERSION:     0.1.7
# CREATED:     2024-03-21 13:31
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Local modules ###
from alcedo import Response, get
from tests import *


def test_alcedo_get_json_endpoint() -> None:
  for _ in range(RUNS):
    response: Response = get(f"{ TEST_ENDPOINT }/json")
    assert response.json() == {"detail": "OK", "float": 1.234, "integer": 200, "null": None}


def test_alcedo_get_orjson_endpoint() -> None:
  for _ in range(RUNS):
    response: Response = get(f"{ TEST_ENDPOINT }/orjson")
    assert response.json() == {"detail": "OK", "float": 1.234, "integer": 200, "null": None}


def test_alcedo_get_plaintext_endpoint() -> None:
  for _ in range(RUNS):
    response: Response = get(f"{ TEST_ENDPOINT }/plaintext")
    assert response.text() == "OK"



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/session/queried_get/bench_aiohttp.py
# VERSION:     0.1.7
# CREATED:     2024-04-01 08:07
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Third-party packages ###
from aiohttp.client import ClientSession
from pytest import mark

### Local modules ###
from tests import *


@mark.aiohttp
@mark.asyncio
async def test_aiohttp_client_session_get_json_endpoint() -> None:
  async with ClientSession() as session:
    for i in range(RUNS):
      async with session.get(
        f"{ TEST_ENDPOINT }/json", params={"foo": "bar", "index": i}
      ) as response:
        assert await response.json() == {
          "detail": "OK",
          "float": 1.234,
          "integer": 200,
          "null": None,
          "query": {"foo": ["bar"], "index": [f"{ i }"]},
        }


@mark.aiohttp
@mark.asyncio
async def test_aiohttp_client_session_get_orjson_endpoint() -> None:
  async with ClientSession() as session:
    for i in range(RUNS):
      async with session.get(
        f"{ TEST_ENDPOINT }/orjson", params={"foo": "bar", "index": i}
      ) as response:
        assert await response.json() == {
          "detail": "OK",
          "float": 1.234,
          "integer": 200,
          "null": None,
          "query": {"foo": ["bar"], "index": [f"{ i }"]},
        }


@mark.aiohttp
@mark.asyncio
async def test_aiohttp_client_session_get_plaintext_endpoint() -> None:
  async with ClientSession() as session:
    for _ in range(RUNS):
      async with session.get(f"{ TEST_ENDPOINT }/plaintext") as response:
        assert await response.text() == "OK"



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/session/post/bench_aiohttp.py
# VERSION:     0.1.7
# CREATED:     2024-03-25 02:00
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from json import dumps

### Third-party packages ###
from aiohttp.client import ClientSession
from pytest import mark

### Local modules ###
from tests import *


@mark.aiohttp
@mark.asyncio
async def test_aiohttp_client_session_post_create_endpoint() -> None:
  async with ClientSession() as session:
    for i in range(RUNS):
      body: dict = {"hello": "world", "count": i}
      async with session.post(f"{ TEST_ENDPOINT }/create", json=dumps(body)) as response:
        assert await response.json() == {"created": body}



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/session/post/bench_requests.py
# VERSION:     0.1.7
# CREATED:     2024-03-27 19:25
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from json import dumps

### Third-party packages ###
from requests import Response, Session
from pytest import mark

### Local modules ###
from tests import *


@mark.requests
def test_requests_session_client_post_endpoint() -> None:
  session: Session = Session()
  for i in range(RUNS):
    body: dict = {"hello": "world", "count": i}
    response: Response = session.post(f"{ TEST_ENDPOINT }/create", json=dumps(body))
    assert response.json() == {"created": body}



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2024 All rights reserved.
# FILENAME:    ~~/tests/session/delete/bench_httpx.py
# VERSION:     0.1.7
# CREATED:     2024-03-28 14:24
# AUTHOR:      Sitt Guruvanich <aekazitt+github@gmail.com>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from json import dumps

### Third-party packages ###
from httpx import Client, Response
from pytest import mark

### Local modules ###
from tests import *


@mark.httpx
def test_httpx_client_delete_endpoint() -> None:
  client: Client = Client()
  for _ in range(RUNS):
    response: Response = client.delete(f"{ TEST_ENDPOINT }/delete")
    assert response.status_code == 204
    assert response.content == b""
